PRO echelle_norm, w,f, fn,fs


  ; Procedure to normalise a 2d echelle frame

  ; Input
  ; w   nw x no  wavelengths
  ; f   nw x no  fluxes

  ; Output
  ; fs  nw x no  function used to rectify f
  ; fn  nw x no  = f / fs


  ; parameters for constructing smoothing filter 

  lam_smooth = 1.4
  ord_smooth = 1.5

  PRINT,'echelle_norm: Normalise echelle spectrum using 2d procedure ...'
  print, 'lam_smooth =',lam_smooth
  print, 'ord_smooth =',ord_smooth

  nw = n_elements(w(*,0))
  nj = n_elements(w(0,*))
  jo = findgen(nj) 
  fb = f
  fk = fb
  fs = fltarr(nw,nj)
  cols = getcolor(/load)


  ; clean CRs from data  -> fb

  dw_filter = 3.0
  fbar = MEAN ( fb )
  FOR j = 0,nj-1 DO BEGIN
    gg = qqsm ( w(*,j),fb(*,j),dw_filter )
    msk = WHERE ( fb(*,j) GT 1.2*gg OR fb(*,j) GT 3*fbar , cnt )
    IF ( cnt GT 0 ) THEN fb(msk,j) = gg(msk)
  ENDFOR 

  window,0,title="-0 raw spectrum (fb)"
  shade_surf,fb ,zrange = [ 0 , max(fb) ]

  ;  smooth over wavelength  -> fs
  FOR j = 0,nj-1 DO fs(*,j) = qqsm ( w(*,j),fb(*,j),lam_smooth )
   window,1,title="-1 masked spectrum (fs)"
   shade_surf,fs,zrange=[0,max(fs)]
   f0 = fs

 
  ; 2d smoothing with Gaussian filter

  ; locate bad orders and replace with local mean, iteratively
  FOR i = 0,nw-1 DO fk(i,*) = qqsm ( jo,fs(i,*),ord_smooth )
  FOR i = 0,nw-1 DO BEGIN
    msk = WHERE ( fs(i,*)/fk(i,*) LT 0.99, cnt )
    IF ( cnt GT 0 ) THEN fs(i,msk) = fk(i,msk)
  ENDFOR
  FOR i = 0,nw-1 DO fk(i,*) = qqsm ( jo,fs(i,*),ord_smooth ) 
  FOR i = 0,nw-1 DO BEGIN
    msk = WHERE ( fs(i,*)/fk(i,*) LT 0.99, cnt )
    IF ( cnt GT 0 ) THEN fs(i,msk) = fk(i,msk)
  ENDFOR
  FOR i = 0,nw-1 DO fk(i,*) = qqsm ( jo,fs(i,*),ord_smooth ) 
  FOR i = 0,nw-1 DO BEGIN
    msk = WHERE ( fs(i,*)/fk(i,*) LT 0.99, cnt )
    IF ( cnt GT 0 ) THEN fs(i,msk) = fk(i,msk)
  ENDFOR
  FOR i = 0,nw-1 DO fk(i,*) = qqsm ( jo,fs(i,*),ord_smooth ) 
  FOR i = 0,nw-1 DO BEGIN
    msk = WHERE ( fk(i,*) LT fs(i,*), cnt )
    IF ( cnt GT 0 ) THEN fk(i,msk) = fs(i,msk)
  ENDFOR

  ; 2d smoothing - Gaussian filter
  FOR j = 0,nj-1 DO fk(*,j) = qqsm ( w(*,j),fk(*,j), lam_smooth )
   
  window,2,title="-2 smoothed masked spectrum (fk)"
  shade_surf,fk,zrange=[0,max(fs)]


  ; global normalisation

  fn = fb / fk
  window,4,title="-4 normalised spectrum (fn = fb / fk)"
  shade_surf,fn,zrange=[0,1.2]

  window,9,title="-9 spectrum, initial mask and final continuum - order by order",xsize=1400,ysize=850
  !p.multi=[0,6,6,0,0]
  FOR j = 0,nj-1 DO BEGIN
  plot,w(*,j),fb(*,j),xstyle=1,ystyle=1,yrange=[0,max(fb)]
  oplot,w(*,j),f0(*,j),color=cols.red
  oplot,w(*,j),fk(*,j),color=cols.green
  ENDFOR
  !p.multi=0      
 
END
